"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.claimCertificates = exports.transferCertificates = exports.issueCertificates = void 0;
const ethers_1 = require("ethers");
const Certificate_1 = require("./Certificate");
const CertificateUtils_1 = require("./CertificateUtils");
async function issueCertificates(certificateInfo, blockchainProperties) {
    const { issuer, registry, activeUser } = blockchainProperties;
    const issuerWithSigner = issuer.connect(activeUser);
    const data = certificateInfo.map((info) => CertificateUtils_1.encodeData({
        generationStartTime: info.generationStartTime,
        generationEndTime: info.generationEndTime,
        deviceId: info.deviceId,
        metadata: info.metadata
    }));
    const batchIssueTx = await issuerWithSigner.issueBatch(certificateInfo.map((info) => info.to), certificateInfo.map((info) => info.amount), data);
    const { events } = await batchIssueTx.wait();
    let issuanceEvent;
    for (const event of events) {
        try {
            issuanceEvent = issuer.interface.parseLog(event);
        }
        catch (e) {
            issuanceEvent = registry.interface.parseLog(event);
        }
        if (issuanceEvent.name === 'IssuanceBatch') {
            break;
        }
    }
    return issuanceEvent.args[2].map((id) => id.toNumber());
}
exports.issueCertificates = issueCertificates;
async function transferCertificates(certificateIds, to, blockchainProperties, from, values) {
    const certificatesPromises = certificateIds.map((certId) => new Certificate_1.Certificate(certId, blockchainProperties).sync());
    const { registry, activeUser } = blockchainProperties;
    const registryWithSigner = registry.connect(activeUser);
    const activeUserAddress = await activeUser.getAddress();
    const fromAddress = from ?? activeUserAddress;
    const certificates = await Promise.all(certificatesPromises);
    const transferTx = await registryWithSigner.safeBatchTransferFrom(fromAddress, to, certificateIds, values ?? certificates.map((cert) => ethers_1.BigNumber.from(cert.owners[fromAddress] ?? 0)), ethers_1.utils.randomBytes(32) // TO-DO: replace with proper data
    );
    await transferTx.wait();
    return transferTx;
}
exports.transferCertificates = transferCertificates;
async function claimCertificates(certificateIds, claimData, blockchainProperties, forAddress, values) {
    const certificatesPromises = certificateIds.map((certId) => new Certificate_1.Certificate(certId, blockchainProperties).sync());
    const certificates = await Promise.all(certificatesPromises);
    const { activeUser, registry } = blockchainProperties;
    const claimer = forAddress ?? (await activeUser.getAddress());
    const encodedClaimData = CertificateUtils_1.encodeClaimData(claimData);
    const data = ethers_1.utils.randomBytes(32);
    const registryWithSigner = registry.connect(activeUser);
    const claimTx = await registryWithSigner.safeBatchTransferAndClaimFrom(claimer, claimer, certificateIds, values ?? certificates.map((cert) => ethers_1.BigNumber.from(cert.owners[claimer] ?? 0)), data, certificates.map(() => encodedClaimData));
    await claimTx.wait();
    return claimTx;
}
exports.claimCertificates = claimCertificates;
//# sourceMappingURL=CertificateBatchOperations.js.map